==> wildfly-clustering-tomcat-17.0.0.Final-source-release.zip: wildfly-clustering-tomcat-17.0.0.Final/common/src/test/java/org/wildfly/clustering/tomcat/AbstractSmokeITCase.java <==
/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2020, Red Hat, Inc., and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.wildfly.clustering.tomcat;

import java.net.URI;
import java.net.URL;
import java.util.Arrays;

import org.apache.hc.client5.http.classic.methods.HttpDelete;
import org.apache.hc.client5.http.classic.methods.HttpGet;
import org.apache.hc.client5.http.classic.methods.HttpHead;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.CloseableHttpResponse;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.HttpStatus;
import org.infinispan.protostream.SerializationContextInitializer;
import org.infinispan.server.test.core.ServerRunMode;
import org.infinispan.server.test.core.TestSystemPropertyNames;
import org.infinispan.server.test.junit4.InfinispanServerRuleBuilder;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Assert;
import org.junit.ClassRule;
import org.junit.rules.TestRule;
import org.wildfly.clustering.marshalling.Externalizer;
import org.wildfly.clustering.tomcat.servlet.MutableIntegerExternalizer;
import org.wildfly.clustering.tomcat.servlet.ServletHandler;
import org.wildfly.clustering.tomcat.servlet.TestSerializationContextInitializer;

/**
 * @author Paul Ferraro
 */
public abstract class AbstractSmokeITCase {

    static final String INFINISPAN_SERVER_HOME = System.getProperty("infinispan.server.home");
    static final String INFINISPAN_SERVER_PROFILE = System.getProperty("infinispan.server.profile");
    static final String INFINISPAN_DRIVER_USERNAME = "testsuite-driver-user";
    static final String INFINISPAN_DRIVER_PASSWORD = "testsuite-driver-password";

    @ClassRule
    public static final TestRule SERVERS = InfinispanServerRuleBuilder.config(INFINISPAN_SERVER_PROFILE)
            .property(TestSystemPropertyNames.INFINISPAN_TEST_SERVER_DIR, INFINISPAN_SERVER_HOME)
            .property("infinispan.client.rest.auth_username", INFINISPAN_DRIVER_USERNAME)
            .property("infinispan.client.rest.auth_password", INFINISPAN_DRIVER_PASSWORD)
            .runMode(ServerRunMode.FORKED)
            .numServers(1)
            .build();

    public static Archive<?> deployment(Class<? extends AbstractSmokeITCase> testClass, Class<? extends ServletHandler<?, ?>> servletClass) {
        return ShrinkWrap.create(WebArchive.class, testClass.getSimpleName() + ".war")
                .addPackage(ServletHandler.class.getPackage())
                .addPackage(servletClass.getPackage())
                .addAsServiceProvider(Externalizer.class, MutableIntegerExternalizer.class)
                .addAsServiceProvider(SerializationContextInitializer.class.getName(), TestSerializationContextInitializer.class.getName() + "Impl")
                ;
    }

    protected void test(URL baseURL1, URL baseURL2) throws Exception {
        URI uri1 = ServletHandler.createURI(baseURL1);
        URI uri2 = ServletHandler.createURI(baseURL2);

        try (CloseableHttpClient client = HttpClients.createDefault()) {
            String sessionId = null;
            int value = 0;
            for (int i = 0; i < 4; i++) {
                for (URI uri : Arrays.asList(uri1, uri2)) {
                    for (int j = 0; j < 4; j++) {
                        try (CloseableHttpResponse response = client.execute(new HttpGet(uri))) {
                            Assert.assertEquals(HttpStatus.SC_OK, response.getCode());
                            Assert.assertEquals(String.valueOf(value++), response.getFirstHeader(ServletHandler.VALUE).getValue());
                            String requestSessionId = response.getFirstHeader(ServletHandler.SESSION_ID).getValue();
                            if (sessionId == null) {
                                sessionId = requestSessionId;
                            } else {
                                Assert.assertEquals(sessionId, requestSessionId);
                            }
                        }
                    }
                    // Grace time between failover requests
                    Thread.sleep(500);
                }
            }
            try (CloseableHttpResponse response = client.execute(new HttpDelete(uri1))) {
                Assert.assertEquals(HttpStatus.SC_OK, response.getCode());
                Assert.assertEquals(sessionId, response.getFirstHeader(ServletHandler.SESSION_ID).getValue());
            }
            try (CloseableHttpResponse response = client.execute(new HttpHead(uri2))) {
                Assert.assertEquals(HttpStatus.SC_OK, response.getCode());
                Assert.assertFalse(response.containsHeader(ServletHandler.SESSION_ID));
            }
        }
    }
}

==> wildfly-clustering-tomcat-18.0.0.Final-source-release.zip: wildfly-clustering-tomcat-18.0.0.Final/common/src/test/java/org/wildfly/clustering/tomcat/AbstractSmokeITCase.java <==
/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2020, Red Hat, Inc., and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.wildfly.clustering.tomcat;

import java.net.URI;
import java.net.URL;
import java.util.Arrays;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;

import org.apache.hc.client5.http.classic.methods.HttpDelete;
import org.apache.hc.client5.http.classic.methods.HttpGet;
import org.apache.hc.client5.http.classic.methods.HttpHead;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.Header;
import org.apache.hc.core5.http.HttpStatus;
import org.infinispan.protostream.SerializationContextInitializer;
import org.infinispan.server.test.core.ServerRunMode;
import org.infinispan.server.test.core.TestSystemPropertyNames;
import org.infinispan.server.test.junit4.InfinispanServerRuleBuilder;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Assert;
import org.junit.ClassRule;
import org.junit.rules.TestRule;
import org.wildfly.clustering.marshalling.Externalizer;
import org.wildfly.clustering.tomcat.servlet.MutableIntegerExternalizer;
import org.wildfly.clustering.tomcat.servlet.ServletHandler;
import org.wildfly.clustering.tomcat.servlet.TestSerializationContextInitializer;

/**
 * @author Paul Ferraro
 */
public abstract class AbstractSmokeITCase {

    static final String INFINISPAN_SERVER_HOME = System.getProperty("infinispan.server.home");
    static final String INFINISPAN_SERVER_PROFILE = System.getProperty("infinispan.server.profile");
    static final String INFINISPAN_DRIVER_USERNAME = "testsuite-driver-user";
    static final String INFINISPAN_DRIVER_PASSWORD = "testsuite-driver-password";

    @ClassRule
    public static final TestRule SERVERS = InfinispanServerRuleBuilder.config(INFINISPAN_SERVER_PROFILE)
            .property(TestSystemPropertyNames.INFINISPAN_TEST_SERVER_DIR, INFINISPAN_SERVER_HOME)
            .property("infinispan.client.rest.auth_username", INFINISPAN_DRIVER_USERNAME)
            .property("infinispan.client.rest.auth_password", INFINISPAN_DRIVER_PASSWORD)
            .runMode(ServerRunMode.FORKED)
            .numServers(1)
            .build();

    public static Archive<?> deployment(Class<? extends AbstractSmokeITCase> testClass, Class<? extends ServletHandler<?, ?>> servletClass) {
        return ShrinkWrap.create(WebArchive.class, testClass.getSimpleName() + ".war")
                .addPackage(ServletHandler.class.getPackage())
                .addPackage(servletClass.getPackage())
                .addAsServiceProvider(Externalizer.class, MutableIntegerExternalizer.class)
                .addAsServiceProvider(SerializationContextInitializer.class.getName(), TestSerializationContextInitializer.class.getName() + "Impl")
                ;
    }

    protected void test(URL baseURL1, URL baseURL2) throws Exception {
        URI uri1 = ServletHandler.createURI(baseURL1);
        URI uri2 = ServletHandler.createURI(baseURL2);

        try (CloseableHttpClient client = HttpClients.createDefault()) {
            AtomicReference<String> sessionId = new AtomicReference<>();
            AtomicInteger value = new AtomicInteger(0);
            for (int i = 0; i < 4; i++) {
                for (URI uri : Arrays.asList(uri1, uri2)) {
                    for (int j = 0; j < 4; j++) {
                        Map.Entry<String, String> result = client.execute(new HttpGet(uri), response -> {
                            Assert.assertEquals(HttpStatus.SC_OK, response.getCode());
                            return Map.entry(response.getFirstHeader(ServletHandler.SESSION_ID).getValue(), response.getFirstHeader(ServletHandler.VALUE).getValue());
                        });
                        Assert.assertEquals(String.valueOf(value.getAndIncrement()), result.getValue());
                        String requestSessionId = result.getKey();
                        if (!sessionId.compareAndSet(null, requestSessionId)) {
                            Assert.assertEquals(sessionId.get(), requestSessionId);
                        }
                    }
                    // Grace time between failover requests
                    Thread.sleep(500);
                }
            }
            String requestSessionId = client.execute(new HttpDelete(uri1), response -> {
                Assert.assertEquals(HttpStatus.SC_OK, response.getCode());
                return response.getFirstHeader(ServletHandler.SESSION_ID).getValue();
            });
            Assert.assertEquals(sessionId.get(), requestSessionId);
            requestSessionId = client.execute(new HttpHead(uri2), response -> {
                Assert.assertEquals(HttpStatus.SC_OK, response.getCode());
                return Optional.ofNullable(response.getFirstHeader(ServletHandler.SESSION_ID)).map(Header::getValue).orElse(null);
            });
            Assert.assertNull(requestSessionId);
        }
    }
}
